##########################
#Author: Chanel Huang
#
#Instead of directly modifying the load function implemented
#in Sage, I decided to write another function, simple_load
# that will load the different filetypes on top of what
# load() already handles. This function will load or open
# the given file by appropriate library.  If it make sense
# for a particular filetype to return an object or some values
# this function will do so.
#
##########################
def simple_load(filename, library = None,attach=False):
    """
    Loads the file given by filename and calls appropriate load or open function
    based on the file extension.  Currently supported: .png, .jpg, .cvs, .mat, .dat, .r, .txt, .sav, .arff, .mtx, .mtz.gz, .gif, .tiff, .bmp, .wav, .npy, .npz and
    those supported by load().  
    
    For some file type, this function returns some value:
        - .cvs : a list of rows in the cvs
        - .png/.jpg: a numpy.ndarray of the image using pyplot
                     a PIL image type when using PIL
        - .xls: a dictionary of worksheet name to list of lines in that worksheet
                only works when the xlrd library is installed
        - .txt: a list of lines in the file
        - .r: execute each line of the script, return nothing
        - .mat: dictionary with variable names as keys, and loaded matrices as values
                 as returned by scipy's loadmat()
        - .arff: a tuple of array of records and MetaData
        - .wav: an int representing the sample rate and
                a numpy array of the sound file
        - .gif, .bit, .tiff: a PIL image object
        - .mtx, .mtz.gz: a matrix described by the file
        - .sav: dictionary object of the IDL save file
        - .f: executes the Fortran file, no return values
        - .dat: a list of lines in the dat file. Each line is further parsed into a list of
                number in each line
                
    INPUT:
        
    - ``filename`` - path to the file to be loaded

    - ``library`` - which library use to load file if there are more than one
                    choice
                  - jpg/png: PIL or matplotlib.pyplot

    - ``attach`` - to attach the file given by URL or not
    
    EXAMPLES::
        
    loading a .csv file::
            
        sage: list = simple_load(DATA+'small.csv')
        sage: list
        
    ::
        
    loading a .png file::
        
        sage: import matplotlib.pyplot
        sage: img = simple_load('http://480.sagenb.org/images/sagenb.png')
        sage: matplotlib.pyplot.imshow(img)
        sage: matplotlib.pyplot.savefig('fig')
        
    ::
    
    loading a .png using PIL:
        
        sage: img = simple_load(DATA+"bakeneko.png", library="PIL")
        sage: img.save('fig.png')
    """
    supported_types = ('.py', '.pyx', '.sage', '.spyx', '.m', '.png', '.jpg', '.cvs', '.mat', '.dat', '.r', '.txt', '.sav', '.arff', '.mtx', '.mtz.gz', '.gif', '.tiff', '.bmp', '.wav', '.npy', '.npz')
    if (not filename.endswith(supported_types)):
        raise ValueError('argument (=%r) to load or attach must have extension %s' %(filename, supported_types))
 
    filename = filename.strip()

    if filename.lower().startswith(('http://', 'https://')):
        if attach:
            # But see http://en.wikipedia.org/wiki/HTTP_ETag for how
            # we will do this.
            # http://www.diveintopython.org/http_web_services/etags.html
            raise NotImplementedError("you can't attach a URL")
        filename = get_remote_file(filename, verbose=False)
    
    if (filename.endswith(('.png', '.jpg')) ):
        if (library == 'PIL'):
            from PIL import Image
            img = Image.open(filename)
        else:
            import matplotlib.pyplot
            img=matplotlib.pyplot.imread(filename)
        return img
    elif (filename.endswith('.csv')):
        import csv
        f = open(filename, 'rU')
        l = []
        try:
            reader = csv.reader(f)
            for row in reader:
                l.append(row)
        finally:
            f.close()
        return l
    elif (filename.endswith('.dat')):
        return numpy.loadtxt(filename)
    elif (filename.endswith('.mat')):
        from scipy import io 
        return io.loadmat(filename)
    elif (filename.endswith('.r')):
        ##any graphs are not displayed
        myfile = open(filename, 'r') 
        lines = myfile.readlines()
        for l in lines:
            r.execute(l)
    elif (filename.endswith('.f')):
        globals()['fortran'](filename)
    elif (filename.endswith('.txt')):
        myfile = open(filename, 'r') 
        lines = myfile.readlines()
        return lines
    elif (filename.endswith('.sav')):
        from scipy import io 
        dictR = dict()
        io.readsav(filename, dictR, True)
        return dictR
    elif (filename.endswith('.arff')):
       from scipy.io import arff
       return arff.loadarff(filename)
    elif (filename.endswith(('.mtx','.mtz.gz'))):
        return scipy.io.mmread(filename)
    elif (filename.endswith(('.gif','.tiff','.bmp')) ):
        from PIL import Image
        return Image.open(filename)
    elif (filename.endswith('.xls')):
        try:
            import xlrd
        except ImportError:
           print "You need to install xlrd to load Excel files"
           return
           
        wb = xlrd.open_workbook(filename)
        dictR = dict()
        for i in range(wb.nsheets):
            sh = wb.sheet_by_index(i)
            l = []
            for row in range(sh.nrows):
                l.append(sh.row_values(row))
            dictR[sh.name] = l
        return dictR
    elif (filename.endswith('.wav')):
        from scipy.io import wavfile
        return wavfile.read(filename)
    elif (filename.endswith(('.npz','.npy'))):
        return numpy.load(filename)
    else:
        load(filename, attach = attach)